/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Hewitt (hewitt@netscape.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== toolbar.css ====================================================
  == Styles used by XUL toolbar-related elements.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: toolbox ::::: */

toolbox {
  -moz-appearance: toolbox;
  background-color: -moz-Dialog;
}

/* ::::: toolbar & menubar ::::: */

toolbar {
  min-width: 1px;
  min-height: 20px;
  border-top: 1px solid ThreeDHighlight;
  border-bottom: 1px solid ThreeDShadow;
}

toolbar[type="menubar"], menubar {
  min-width: 1px;
  min-height: 20px;
  border-bottom: 1px solid ThreeDShadow;
  border-top: 0px !important;
}

/* ::::: toolbar decorations ::::: */

toolbarseparator {
  margin : 2px 0.2em;
  border-right : 1px solid ThreeDHighlight;
  border-left : 1px solid ThreeDShadow;
  width : 2px;
}

toolbarspacer {
  width: 15px;
}

/* ::::: toolbarpaletteitem ::::: */

toolbarpaletteitem {
  cursor: -moz-grab;
}

.toolbarpaletteitem-box[type="spacer"],
.toolbarpaletteitem-box[type="spring"] {
  border: 1px solid #808080;
  background-color: #FFF !important;
}

toolbarpaletteitem[place="toolbar"] {
  margin-left: -2px;
  margin-right: -2px;
  border-left: 2px solid transparent;
  border-right: 2px solid transparent;
}

toolbarpaletteitem[place="toolbar"] > toolbarspacer {
  width: 11px;
}

.toolbarpaletteitem-box[type="spacer"][place="toolbar"],
.toolbarpaletteitem-box[type="spring"][place="toolbar"] {
  margin-top: 2px;
  margin-bottom: 2px;
  -moz-margin-start: 0px;
  -moz-margin-end: 2px;
}

.toolbarpaletteitem-box[type="separator"][place="palette"] {
  width: 2px;
  height: 50px;
}

.toolbarpaletteitem-box[type="spacer"][place="palette"],
.toolbarpaletteitem-box[type="spring"][place="palette"] {
  margin-bottom: 2px;
  width: 50px;
  height: 50px;
}

.toolbarpaletteitem-box[type="spring"][place="palette"] {
  background: url("chrome://global/skin/toolbar/spring.gif") no-repeat center;
}

/* ..... drag and drop feedback ..... */

toolbarpaletteitem[dragover="left"] {
  border-left-color: #000000;
}

toolbarpaletteitem[dragover="right"] {
  border-right-color: #000000;
}
